import {Context} from 'koa';
import {promises, createReadStream} from "fs";
import {createReadStreamFromZip, entriesByNaturalOrder, readZip} from "../util/ziputil";

function since_last_modified(ctx: Context, last_modified: Date): boolean{
    const con = ctx.get("If-Modified-Since");
    if(con === "") return false;
    const mdate = new Date(con);
    if(last_modified > mdate) return false;
    ctx.status = 304;
    return true;
}

export async function renderZipImage(ctx: Context,path : string, page:number){
    const image_ext = ['gif', 'png', 'jpeg', 'bmp', 'webp', 'jpg'];
    let zip = await readZip(path);
    const entries = entriesByNaturalOrder(zip).filter(x=>{
        const ext = x.name.split('.').pop();
        return ext !== undefined && image_ext.includes(ext);
    });
    if(0 <= page && page < entries.length){
        const entry = entries[page];
        const last_modified = new Date(entry.time);
        if(since_last_modified(ctx,last_modified)){
            return;
        }
        const read_stream = (await createReadStreamFromZip(zip,entry));
        read_stream.on('close',()=>zip.close());
        ctx.body = read_stream;
        ctx.response.length = entry.size;
        //console.log(`${entry.name}'s ${page}:${entry.size}`);
        ctx.response.type = entry.name.split(".").pop() as string;
        ctx.status = 200;
        ctx.set('Date', new Date().toUTCString());
        ctx.set("Last-Modified",last_modified.toUTCString());
    }
    else{
        ctx.status = 404;
    }
}
export async function renderImage(ctx: Context,path : string){
    const ext = path.trim().split('.').pop();
    if(ext === undefined) { 
        ctx.status = 404;
        return;
    }
    ctx.response.type = ext;
    ctx.body = createReadStream(path);
}

export async function renderVideo(ctx: Context,path : string){
    const ext = path.trim().split('.').pop();
    if(ext === undefined) { 
        //ctx.status = 404;
        console.error(`${path}:${ext}`)
        return;
    }
    ctx.response.type = ext;
    const range_text = ctx.request.get("range");
    const stat = await promises.stat(path);
    let start = 0;
    let end = 0;
    ctx.set('Last-Modified',(new Date(stat.mtime).toUTCString()));
    ctx.set('Date', new Date().toUTCString());
    ctx.set("Accept-Ranges", "bytes");
    if(range_text === ''){
        end = 1024*512;
        end = Math.min(end,stat.size-1);
        if(start > end){
            ctx.status = 416;
            return;
        }
        ctx.status = 200;
        ctx.length = stat.size;
        let stream = createReadStream(path);
        ctx.body = stream;
    }
    else{
        const m = range_text.match(/^bytes=(\d+)-(\d*)/);
        if(m === null){
            ctx.status = 416;
            return;
        }
        start = parseInt(m[1]);
        end = m[2].length > 0 ? parseInt(m[2]) : start + 1024*1024;
        end = Math.min(end,stat.size-1);
        if(start > end){
            ctx.status = 416;
            return;
        }
        ctx.status = 206;
        ctx.length = end - start + 1;
        ctx.response.set("Content-Range",`bytes ${start}-${end}/${stat.size}`);
        ctx.body = createReadStream(path,{
            start:start,
            end:end
        });//inclusive range.
    }
}